/**
 @file BMSFile.h
 @brief BMS 노트/파일 인터페이스 선언
 
 Created by Hoppang on 08. 12. 20.
 */

#import <Cocoa/Cocoa.h>
#import <QTKit/QTKit.h>
#include </Developer/FMOD Programmers API Mac/api/inc/fmod.h>

@class EDController;

/**
 @brief BMS 노트 객체
 */
@interface BMSNote : NSObject {
	FMOD_SOUND *audio;					///< fmod 소리 파일 정보
	CFAbsoluteTime appearanceTime;		///< 노트가 등장하는 시간
	unsigned int indexOfInstrument;		///< 악기 번호, 또는 BGA 번호
	unsigned int length;				///< (롱노트일 때만) 노트 지속시간
	unsigned int type;					///< 노트 종류(오브젝트, 이벤트)
	unsigned int channel;				///< 채널(배경음 아닌 오브젝트에만 유효)
	unsigned int track;					///< 마디
}

@property CFAbsoluteTime appearanceTime;
@property unsigned int indexOfInstrument;
@property unsigned int length;
@property unsigned int type;
@property unsigned int channel;
@property unsigned int track;
@property(nonatomic) FMOD_SOUND *audio;

@end

/**
 @brief BMS 파일 객체
 */
@interface BMSFile : NSObject {
	int BPM;						///< Beat Per Minute - BMS에서는 4분당 재생되는 마디 수
	unsigned int difficulty;		///< 제작자가 지정한 난이도
	unsigned int numberOfPlayer;	///< 플레이어수(거의 1)
	unsigned int playLevel;			///< 난이도. difficulty와 어떻게 다른지는 잘 모르겠다
	unsigned int trackLength;		///< 마디당 길이(?)
	unsigned int length;			///< 총 길이

	NSString *artist;				///< 제작자
	NSString *genre;				///< 장르
	NSString *title;				///< 곡이름
	NSString *stageFile;			///< 로딩 화면에 표시할 그림파일
	NSString *banner;				///< 배너(?)

	NSString *bmspath;				///< BMS 파일 경로
	NSString *filename;				///< BMS 파일 이름

	NSArray *notes;					///< 노트
	FMOD_SOUND *audioFiles[255];	///< 소리 파일
	NSString *animationFiles[255];	///< 수정한 그림 파일
	NSArray *arrayOfBmsTextLines;
	
	EDController *edController;		///< 관리자 객체
	double timePerTrack;			///< 마디당 시간(초단위)
}

- (id)initWithFile:(NSString *)pathOfBms edController:(EDController *)aEDController;
- (void)_parse;
- (NSArray *)notesWithTimeFrom:(CFAbsoluteTime)fromTime to:(CFAbsoluteTime)toTime;
- (FMOD_SOUND **)audioFiles;

@property(readonly, nonatomic, retain) NSString *stageFile;
@property(nonatomic, retain) NSString *bmspath;
@property(readonly) unsigned int length;

@end
