/**
 @file EDController.m
 @brief EgyptDJ ~(-_-)~ 컨트롤러 소스 파일
 
 Created by Hoppang on 08. 12. 20.
 */

#import "EDController.h"
#import "EDView.h"
#import "BMSFile.h"
#import "EDModeManager.h"
#import "PlayingMode.h"
#import "MenuMode.h"

@implementation EDController

@synthesize soundSystem;

- (id)init
{
	FMOD_RESULT result;
	unsigned int fmodVersion;
	
	if( (self = [super init]) )
	{
		result = FMOD_System_Create(&soundSystem);

		result = FMOD_System_GetVersion(soundSystem, &fmodVersion);
		if (fmodVersion < FMOD_VERSION)
		{
			NSLog(@"You are using an old version of FMOD %08x.  This program requires %08x\n", fmodVersion, FMOD_VERSION);
			exit(1);
		}
		
		result = FMOD_System_Init(soundSystem, 32, FMOD_INIT_NORMAL, NULL);

		playingMode = [[PlayingMode alloc] init];
		menuMode = [[MenuMode alloc] init];
	}
	return self;
}

- (void)awakeFromNib
{
	const double FrameRate = 24.0;
	bms = [[BMSFile alloc] initWithFile:@"/Users/hoppang/Downloads/BMS/Orthodox/Orthodox [5K].bms"
						   edController:self];
	modeManager = [[EDModeManager alloc] initWithView:view];
	[modeManager setRunningMode:playingMode];

	// 타이머 생성
	timer = [[NSTimer scheduledTimerWithTimeInterval:1.0/FrameRate
											  target:self
											selector:@selector(run:)
											userInfo:nil
											 repeats:YES] retain];

	startTime = CFAbsoluteTimeGetCurrent();
}

/**
 @brief 주 실행 루프. BMS 플레이중(?)
 @param aTimer 타이머
 */
- (void)run:(NSTimer *)aTimer
{
	static CFAbsoluteTime fromTime = 0;
	CFAbsoluteTime toTime = CFAbsoluteTimeGetCurrent() - startTime;
	
	int channelsPlaying = 0;
	FMOD_RESULT result;
	FMOD_CHANNEL *channel = 0;
	NSArray *noteToPlay = [bms notesWithTimeFrom:fromTime to:toTime];
	if([noteToPlay count])
	{
		fromTime = toTime;
		id object;
		NSEnumerator *e = [noteToPlay objectEnumerator];
		while( (object = [e nextObject]) )
		{
			FMOD_SOUND *s = (FMOD_SOUND *)[object audio];
			result = FMOD_System_PlaySound([self soundSystem], FMOD_CHANNEL_FREE,
										   s, 0, &channel);
			FMOD_System_GetChannelsPlaying([self soundSystem], &channelsPlaying);
			NSLog(@"audio = %x, playsound result = %d, playing = %d", s, result, channelsPlaying);
		}
	}
	// 재생이 끝난 채널을 정리해주고 기타등등 여러 좋은일을 하시는 update
	FMOD_System_Update([self soundSystem]);

	[modeManager update];
	[modeManager render:view];

	[view setNeedsDisplay:YES];
}

- (void)keyDown:(NSEvent *)event 
{
	NSLog(@"KeyDown:");
}

// 최초 반응자 만들기 (대리자 메서드)
- (BOOL)acceptsFirstResponder
{
	return YES;	// 최초 반응자가 되는 것을 허용
}

@end
